@@ -41,6 +41,9 @@
     >         case myFunction input of
     >             Nothing -> return () -- Can happen
     >             Just output -> output `shouldSatisfy` isValid
+
+    Definitely also look at the genvalidity-property and genvalidity-hspec packages
+    for more info on how to use this package.
     -}
 {-# LANGUAGE DefaultSignatures #-}
 {-# LANGUAGE FlexibleContexts #-}
@@ -71,6 +74,9 @@ module Data.GenValidity
     , shrinkValidStructurallyWithoutExtraFiltering
     , module Data.GenValidity.Utils
 
+    -- * Strange, possibly useful functions
+    , genUtf16SurrogateCodePoint
+
     -- * Re-exports
     , module Data.Validity
 
@@ -95,7 +101,6 @@ import Data.Validity
 import Data.Fixed (Fixed(..), HasResolution)
 #if MIN_VERSION_base(4,9,0)
 import Data.List.NonEmpty (NonEmpty((:|)))
-import qualified Data.List.NonEmpty as NE
 #endif
 #if MIN_VERSION_base(4,8,0)
 import Data.Word (Word8, Word16, Word32, Word64)
@@ -103,9 +108,11 @@ import Data.Word (Word8, Word16, Word32, Word64)
 import Data.Word (Word, Word8, Word16, Word32, Word64)
 #endif
 import Data.Int (Int8, Int16, Int32, Int64)
+import Data.Char (chr)
 import Data.Ratio ((%))
 import GHC.Generics
 import GHC.Real (Ratio(..))
+import Control.Monad (guard)
 
 import Test.QuickCheck hiding (Fixed)
 
@@ -144,6 +151,9 @@ import Data.GenValidity.Utils
 -- > instance GenUnchecked MyType where
 -- >     genUnchecked = MyType <$> genUnchecked <*> genUnchecked
 --
+-- If this is not possible because there is no 'GenUnchecked' instance available for one of the
+-- sub-parts of your type, __then do not instantiate 'GenUnchecked' for your type__.
+-- Just continue with 'GenValid' instead.
 --
 -- __Step 2__: If an instatiation via 'Generic' is not possible, then you should emulate what
 --         'genericGenUnchecked' does.
@@ -153,11 +163,11 @@ import Data.GenValidity.Utils
 --
 -- === Warning: Invalid values can be funky
 --
--- Some types have serious validity constraints. See 'Text' or 'ByteString' for example.
+-- Some types have serious validity constraints. See 'Rational' for example.
 -- These can behave very strangely when they are not valid.
 -- In that case, __do not override 'GenUnchecked' such that 'genUnchecked' only generates valid values__.
 -- In that case, do not override 'genUnchecked' at all.
--- Instead, use 'genValid' from 'GenValid' (see below) instead.
+-- Instead, use 'genValid' from 'GenValid' (see below) instead and consider not instantiating 'GenUnchecked' at all.
 class GenUnchecked a where
     genUnchecked :: Gen a
     default genUnchecked :: (Generic a, GGenUnchecked (Rep a)) =>
@@ -175,6 +185,8 @@ class GenUnchecked a where
 -- === How to instantiate 'GenValid'
 --
 -- __Step 1__: Try to instantiate 'GenValid' without overriding any functions.
+--             This is only possible if your type has a 'GenUnchecked' instance.
+--             If it doesn't, go to step 2.
 --             It is possible that, if few values are valid or if validity
 --             checking is expensive, that the resulting generator is too slow.
 --             In that case, go to Step 2.
@@ -196,8 +208,7 @@ class GenUnchecked a where
 --
 -- > arbitrary = genValid
 -- > shrink = shrinkValid
-class (Validity a, GenUnchecked a) =>
-      GenValid a where
+class Validity a => GenValid a where
     -- | Generate a valid datum, this should cover all possible valid values in
     -- the type
     --
@@ -209,6 +220,7 @@ class (Validity a, GenUnchecked a) =>
     -- If you do, make sure that it is possible to generate all possible valid
     -- data, otherwise your testing may not cover all cases.
     genValid :: Gen a
+    default genValid :: GenUnchecked a => Gen a
     genValid = genUnchecked `suchThat` isValid
 
     -- | Shrink a valid value.
@@ -222,6 +234,7 @@ class (Validity a, GenUnchecked a) =>
     -- might fail for a different reason than for the reason that it originally failed.
     -- This would lead to very confusing error messages.
     shrinkValid :: a -> [a]
+    default shrinkValid :: GenUnchecked a => a -> [a]
     shrinkValid = filter isValid . shrinkUnchecked
 
 -- | A class of types for which invalid values can be generated.
@@ -237,8 +250,7 @@ class (Validity a, GenUnchecked a) =>
 --             for parametric values.
 --
 -- __Step 2__: Instantiate 'GenInvalid' without overriding any functions.
-class (Validity a, GenUnchecked a) =>
-      GenInvalid a where
+class Validity a => GenInvalid a where
     genInvalid :: Gen a
     -- | Generate an invalid datum, this should cover all possible invalid
     -- values
@@ -248,9 +260,11 @@ class (Validity a, GenUnchecked a) =>
     -- To speed up testing, it may be a good idea to implement this yourself.
     -- If you do, make sure that it is possible to generate all possible
     -- invalid data, otherwise your testing may not cover all cases.
+    default genInvalid :: GenUnchecked a => Gen a
     genInvalid = genUnchecked `suchThat` isInvalid
 
     shrinkInvalid :: a -> [a]
+    default shrinkInvalid :: GenUnchecked a => a -> [a]
     shrinkInvalid = filter isInvalid . shrinkUnchecked
 
 instance (GenUnchecked a, GenUnchecked b) => GenUnchecked (a, b) where
@@ -271,11 +285,9 @@ instance (GenValid a, GenValid b) => GenValid (a, b) where
             a <- resize r genValid
             b <- resize s genValid
             return (a, b)
-    shrinkValid (a, b) = ((,) <$> shrinkValid a <*> shrinkValid b)
-      ++ [ (a', b) | a' <- shrinkValid a ]
-      ++ [ (a, b') | b' <- shrinkValid b ]
+    shrinkValid = shrinkTuple shrinkValid shrinkValid
 
-instance (GenInvalid a, GenInvalid b) => GenInvalid (a, b) where
+instance (GenUnchecked a, GenInvalid a, GenUnchecked b, GenInvalid b) => GenInvalid (a, b) where
     genInvalid =
         sized $ \n -> do
             (r, s) <- genSplit n
@@ -301,6 +313,8 @@ instance (GenValid a, GenValid b) => GenValid (Either a b) where
 -- | This instance ensures that the generated tupse contains at least one invalid element. The other element is unchecked.
 instance (GenInvalid a, GenInvalid b) => GenInvalid (Either a b) where
     genInvalid = oneof [Left <$> genInvalid, Right <$> genInvalid]
+    shrinkInvalid (Left v) = Left <$> shrinkInvalid v
+    shrinkInvalid (Right v) = Right <$> shrinkInvalid v
 
 instance (GenUnchecked a, GenUnchecked b, GenUnchecked c) =>
          GenUnchecked (a, b, c) where
@@ -330,7 +344,8 @@ instance (GenValid a, GenValid b, GenValid c) => GenValid (a, b, c) where
         ]
 
 -- | This instance ensures that the generated triple contains at least one invalid element. The other two are unchecked.
-instance (GenInvalid a, GenInvalid b, GenInvalid c) =>
+instance ( GenUnchecked a, GenUnchecked b, GenUnchecked c
+         , GenInvalid a, GenInvalid b, GenInvalid c) =>
          GenInvalid (a, b, c) where
     genInvalid =
         sized $ \n -> do
@@ -381,7 +396,8 @@ instance (GenValid a, GenValid b, GenValid c, GenValid d) =>
         ]
 
 -- | This instance ensures that the generated triple contains at least one invalid element. The other two are unchecked.
-instance (GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d) =>
+instance ( GenUnchecked a, GenUnchecked b, GenUnchecked c, GenUnchecked d
+         , GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d) =>
          GenInvalid (a, b, c, d) where
     genInvalid =
         sized $ \n -> do
@@ -442,7 +458,8 @@ instance (GenValid a, GenValid b, GenValid c, GenValid d, GenValid e) =>
         ]
 
 -- | This instance ensures that the generated triple contains at least one invalid element. The other two are unchecked.
-instance (GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d, GenInvalid e) =>
+instance ( GenUnchecked a, GenUnchecked b, GenUnchecked c, GenUnchecked d, GenUnchecked e
+         , GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d, GenInvalid e) =>
          GenInvalid (a, b, c, d, e) where
     genInvalid =
         sized $ \n -> do
@@ -493,30 +510,20 @@ instance GenValid a => GenValid (Maybe a) where
 
 instance GenInvalid a => GenInvalid (Maybe a) where
     genInvalid = Just <$> genInvalid
+    shrinkInvalid Nothing = [] -- Should not happen
+    shrinkInvalid (Just a) = Just <$> shrinkInvalid a
 
 #if MIN_VERSION_base(4,9,0)
 instance GenUnchecked a => GenUnchecked (NonEmpty a) where
-    genUnchecked = do
-      l <- genUnchecked
-      case NE.nonEmpty l of
-        Nothing -> scale (+1) genUnchecked
-        Just ne -> pure ne
+    genUnchecked = genNonEmptyOf genUnchecked
     shrinkUnchecked (v :| vs) = [ e :| es | (e, es) <- shrinkUnchecked (v, vs)]
 
 instance GenValid a => GenValid (NonEmpty a) where
-    genValid = do
-      l <- genValid
-      case NE.nonEmpty l of
-        Nothing -> scale (+1) genValid
-        Just ne -> pure ne
+    genValid = genNonEmptyOf genValid
     shrinkValid (v :| vs) = [ e :| es | (e, es) <- shrinkValid (v, vs)]
 
-instance GenInvalid a => GenInvalid (NonEmpty a) where
-    genInvalid = do
-      l <- genInvalid
-      case NE.nonEmpty l of
-        Nothing -> scale (+1) genInvalid
-        Just ne -> pure ne
+instance (GenUnchecked a, GenInvalid a) => GenInvalid (NonEmpty a) where
+    genInvalid = genNonEmptyOf genInvalid
 #endif
 
 instance GenUnchecked a => GenUnchecked [a] where
@@ -531,7 +538,7 @@ instance GenValid a => GenValid [a] where
 
 -- | This instance ensures that the generated list contains at least one element
 -- that satisfies 'isInvalid'. The rest is unchecked.
-instance GenInvalid a => GenInvalid [a] where
+instance (GenUnchecked a, GenInvalid a) => GenInvalid [a] where
     genInvalid =
         sized $ \n -> do
             (x, y, z) <- genSplit3 n
@@ -544,89 +551,119 @@ instance GenUnchecked () where
     genUnchecked = arbitrary
     shrinkUnchecked = shrink
 
-instance GenValid ()
+instance GenValid () where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Bool where
     genUnchecked = arbitrary
     shrinkUnchecked = shrink
 
-instance GenValid Bool
+instance GenValid Bool where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Ordering where
     genUnchecked = arbitrary
     shrinkUnchecked = shrink
 
-instance GenValid Ordering
+instance GenValid Ordering where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Char where
-    genUnchecked = arbitrary
+    genUnchecked = frequency [(9, choose (minBound, maxBound)), (1, genUtf16SurrogateCodePoint)]
     shrinkUnchecked = shrink
 
-instance GenValid Char
+genUtf16SurrogateCodePoint :: Gen Char
+genUtf16SurrogateCodePoint = chr <$> oneof [choose (0xD800, 0xDBFF), choose (0xDC00, 0xDFFF)]
+
+instance GenValid Char where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Int where
-    genUnchecked = arbitrary
+    genUnchecked = genIntX
     shrinkUnchecked = shrink
 
-instance GenValid Int
+instance GenValid Int where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
+
 instance GenUnchecked Int8 where
-    genUnchecked = arbitrary
+    genUnchecked = genIntX
     shrinkUnchecked = shrink
 
-instance GenValid Int8
+instance GenValid Int8 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Int16 where
-    genUnchecked = arbitrary
+    genUnchecked = genIntX
     shrinkUnchecked = shrink
 
-instance GenValid Int16
+instance GenValid Int16 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Int32 where
-    genUnchecked = arbitrary
+    genUnchecked = genIntX
     shrinkUnchecked = shrink
 
-instance GenValid Int32
+instance GenValid Int32 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Int64 where
-    genUnchecked = arbitrary
+    genUnchecked = genIntX
     shrinkUnchecked = shrink
 
 instance GenValid Int64 where
-    genValid = arbitrary
-    shrinkValid = shrink
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Word where
-    genUnchecked = arbitrary
+    genUnchecked = genWordX
     shrinkUnchecked = shrink
 
-instance GenValid Word
+instance GenValid Word where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Word8 where
-    genUnchecked = arbitrary
+    genUnchecked = genWordX
     shrinkUnchecked = shrink
 
-instance GenValid Word8
+instance GenValid Word8 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Word16 where
-    genUnchecked = arbitrary
+    genUnchecked = genWordX
     shrinkUnchecked = shrink
 
-instance GenValid Word16
+instance GenValid Word16 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Word32 where
-    genUnchecked = arbitrary
+    genUnchecked = genWordX
     shrinkUnchecked = shrink
 
-instance GenValid Word32
+instance GenValid Word32 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Word64 where
-    genUnchecked = arbitrary
+    genUnchecked = genWordX
     shrinkUnchecked = shrink
 
-instance GenValid Word64
+instance GenValid Word64 where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Float where
-    genUnchecked = frequency [(9, arbitrary), (1, elements [read "NaN", read "Infinity", read "-Infinity", read "-0"])]
+    genUnchecked = genFloat
 #if MIN_VERSION_QuickCheck(2,9,2)
     shrinkUnchecked f = if
       | isInfinite f -> []
@@ -641,7 +678,7 @@ instance GenValid Float where
     shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Double where
-    genUnchecked = frequency [(9, arbitrary), (1, elements [read "NaN", read "Infinity", read "-Infinity", read "-0"])]
+    genUnchecked = genDouble
 #if MIN_VERSION_QuickCheck(2,9,2)
     shrinkUnchecked d = if
       | isInfinite d -> []
@@ -656,7 +693,7 @@ instance GenValid Double where
     shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Integer where
-    genUnchecked = arbitrary
+    genUnchecked = genInteger
     shrinkUnchecked = shrink
 
 instance GenValid Integer
@@ -664,8 +701,7 @@ instance GenValid Integer
 #if MIN_VERSION_base(4,8,0)
 instance GenUnchecked Natural where
     genUnchecked = fromInteger . abs <$> genUnchecked
-    shrinkUnchecked 0 = []
-    shrinkUnchecked n = [0 .. n-1]
+    shrinkUnchecked = fmap (fromInteger . abs) . shrinkUnchecked . toInteger
 
 instance GenValid Natural where
     genValid = fromInteger . abs <$> genValid
@@ -676,10 +712,18 @@ instance (Integral a, GenUnchecked a) => GenUnchecked (Ratio a) where
     shrinkUnchecked (n :% d) = [n' :% d' | (n', d') <- shrinkUnchecked (n, d)]
 
 instance (Integral a, Num a, Ord a, GenValid a) => GenValid (Ratio a) where
-    genValid = (%) <$> genValid <*> (genValid `suchThat` (> 0))
-    shrinkValid (n :% d) = [n' % d' | (n', d') <- shrinkValid (n, d), d' > 0]
+    genValid = (do
+      n <- genValid
+      d <- (genValid `suchThat` (> 0))
+      pure $ n :% d) `suchThat` isValid
+    shrinkValid (n :% d) = do
+      (n', d') <- shrinkValid (n, d)
+      guard $ d' > 0
+      let candidate = n' :% d'
+      guard $ isValid candidate
+      pure $ n' % d'
 
-instance (Integral a, Num a, Ord a, GenValid a) => GenInvalid (Ratio a)
+instance (Integral a, Num a, Ord a, Validity a, GenUnchecked a) => GenInvalid (Ratio a)
 
 instance HasResolution a => GenUnchecked (Fixed a) where
     genUnchecked = MkFixed <$> genUnchecked
